c     
c     "lcs.f" 
c     
c     First order Godunov-type 1D scheme;
c     Uses Linearized Characteristic-Based Appr. Riemann Solver (LCS);
c     Written by  Andrei A. Chernousov;
c     E-mail: <andrei99@iname.com>
c     http://www.geocities.com/andrei_chernousov
c     November 12, 2001
c     

      PROGRAM LCS
      
c     
c     Definitions      
c     
      INTEGER I, J
c     Initial variables
      INTEGER LEN, NSTEP
      REAL DX, DT
c     Number of current step
      INTEGER STEP
c     Ratio DT/DX
      REAL DTDX
c     Primitive variables
      REAL P, R, U
c     Arrays of conservative variables in cells and arrays for fluxes
      INTEGER MAXLEN
      PARAMETER (MAXLEN = 10000)
      REAL U1(MAXLEN+2), U2(MAXLEN+2), U3(MAXLEN+2)
      REAL F1(MAXLEN+1), F2(MAXLEN+1), F3(MAXLEN+1)
c     Physical properties for gas: ratio of spec. heats, etc.
      REAL K, KM1, KDKM1, KD4, KM1D2, KD2
      PARAMETER(K = 1.4)
      PARAMETER(KM1 = K - 1., KDKM1 = K/(K - 1.), KD4 = K/4.)
      PARAMETER(KM1D2 = (K - 1.)/2, KD2 = K/2.)
c     Variables for Linearized Characteristic-based Solver (LCS)
      REAL P1, R1, V1, C1, P2, R2, V2, C2
      REAL JP, JM, JO, ALP, ALM, ALO
      REAL JP1, JM1, JO1, ALP1, ALM1, ALO1
      REAL JP2, JM2, JO2, ALP2, ALM2, ALO2
      REAL CP, CM, CO

      
c     
c     Reading initial data from "lcs.ini" 
c     

c     Open "lcs.ini" file
      OPEN(UNIT=9, FILE='lcs.ini', STATUS='OLD');
c     Read cells count - LEN - and check!
      READ(UNIT=9, FMT=*) LEN
c     WRITE(*, *) 'LEN =', LEN      
      IF(LEN .GT. MAXLEN) THEN
         WRITE(*, *) 'LEN is too big; increase parameter MAXLEN!'
         STOP 
      ENDIF
c     Read cell length - deltaX
      READ(UNIT=9, FMT=*) DX
c     WRITE(*, *) 'deltaX = ', DX
c     Read step in time - deltaT  
      READ(UNIT=9, FMT=*) DT
c     WRITE(*, *) 'deltaT = ', DT
c     Read overall step count - numstep
      READ(UNIT=9, FMT=*) NSTEP
c     WRITE(*, *) 'numstep = ', NSTEP
c     Close "lcs.ini" file
      CLOSE(9)
c     Compute and store DT / DX 
      DTDX = DT / DX


c     
c     Setting initial conditions
c     

c     Primitive variables in the left half
      R = 1.0
      P = 100000.0
      U = 0.0
c     Fill up cells in the left half
      DO 10 I = 2, LEN/2
         U1(I) = R
         U2(I) = R * U
         U3(I) = (P / KM1 + 0.5 * R * U**2)
 10   CONTINUE
c     Primitive variables in the right half
      R = 2.0
      P = 200000.0
      U = 0.0
c     Fill up cells in the right half
      DO 20 I = LEN/2+1, LEN+1
         U1(I) = R
         U2(I) = R * U
         U3(I) = (P / KM1 + 0.5 * R * U**2)
 20   CONTINUE


c     
c     Numerical solution procedure - numstep timesteps  
c     

c     Main loop over NSTEP steps
      DO 50 STEP = 1, NSTEP
         
c     Printing current step
c     WRITE(*,*) STEP
         
c     Boundary conditions in 'ghost cells': reflective BC
         U1(1) =  U1(2)
         U2(1) = -U2(2)
         U3(1) =  U3(3)
         U1(LEN+2) =  U1(LEN+1)
         U2(LEN+2) = -U2(LEN+1)
         U3(LEN+2) =  U3(LEN+1)                  
         
c     'Fluxes' stage: evaluation of fluxes at cells' interfaces
         DO 30 I = 1, LEN+1
            
c     c        decode 'left' parameters and compute 'invariants'
            R1 =  U1(I);
            V1 =  U2(I) / R1;
            P1 = (U3(I) - 0.5 * R1 * V1**2) * KM1
            C1 = SQRT(K * P1 / R1)
            JO1 = P1 - C1 * C1 * R1;
            JP1 = V1 + P1 / (R1 * C1);
            JM1 = V1 - P1 / (R1 * C1);
            
c     c        decode 'right' parameters and compute 'invariants'
            R2 =  U1(I+1);
            V2 =  U2(I+1) / R2;
            P2 = (U3(I+1) - 0.5 * R2 * V2**2) * KM1
            C2 = SQRT(K * P2 / R2)
            JO2 = P2 - C2 * C2 * R2
            JP2 = V2 + P2 / (R2 * C2)
            JM2 = V2 - P2 / (R2 * C2)
            
c     c        initial guesses for speeds: velocity and speed of sound
            U = 0.25 * (JP1 + JM1 + JP2 + JM2)
            C =  KD4 * (JP1 - JM1 + JP2 - JM2)
            
c     c        initial guess for characteristics speeds
            CP = U + C
            CM = U - C
            CO = U
            
c     c        choose ALs and Js
            IF (CP .GT. 0.) THEN 
               ALP = KM1D2 * (JM1 - JP1) / JO1 
               JP  = JP1 
            ELSE 
               ALP = KM1D2 * (JM2 - JP2) / JO2 
               JP  = JP2
            ENDIF
            IF (CM .GT. 0.) THEN 
               ALM = KM1D2 * (JP1 - JM1) / JO1 
               JM  = JM1 
            ELSE
               ALM = KM1D2 * (JP2 - JM2) / JO2
               JM  = JM2
            ENDIF
            IF (CO .GT. 0.) THEN 
               ALO = KD2 * (JP1 - JM1) 
               ALO = - (ALO * ALO)
               JO  = JO1
            ELSE
               ALO = KD2 * (JP2 - JM2) 
               ALO = - (ALO * ALO)
               JO  = JO2
            ENDIF	          
            
c     c        decode parameters at interface
            P = (JP - JM) / (ALP - ALM)
            U = JP - ALP * P
            R = (JO - P) / ALO          
            
c     c        compute fluxes
            F1(I) = R * U
            F2(I) = R * U * U + P
            F3(I) = U * (KDKM1 * P + 0.5 * R * U**2)

c     End of fluxes evaluation loop
 30      CONTINUE
         
c     'Evolution' stage: updating conservative variables in cells
         DO 40 I = 2, LEN+1
            U1(I) = U1(I) + DTDX * (F1(I-1) - F1(I))
            U2(I) = U2(I) + DTDX * (F2(I-1) - F2(I))
            U3(I) = U3(I) + DTDX * (F3(I-1) - F3(I))
 40      CONTINUE
         
c     End of main loop on NSTEP steps
 50   CONTINUE


c     
c     Writing solution arrays to file "output.dat"
c     

c     Open file "output.dat"
      OPEN(UNIT=9, FILE='output.dat', STATUS='UNKNOWN');
c     Write line count and data rows count
c     WRITE(9, *) LEN+2
c     WRITE(9, *) 3
c     Write first scaling line
      WRITE(9, FMT=70) 0., ' ', 90000.0, ' ', 0.9, ' ', 0. 
c     'Decode' parameters and write to file
      DO 60 I = 2, LEN+1
         R =  U1(I);
         U =  U2(I) / R;
         P = (U3(I) - 0.5 * R * U**2) * KM1
         WRITE(9, FMT=70) (I-1.5)*DX, ' ', P, ' ', R, ' ', U 
 60   CONTINUE
c     Write final scaling line
      WRITE(9, FMT=70) LEN*DX, ' ', 210000., ' ', 2.1, ' ', 100.
 70   FORMAT(1X, F8.4, A1, F7.0, A1, F5.3, A1, F7.3)	
c     Close file "output.dat"
      CLOSE(9)

      STOP
      END
